@@ -85,6 +85,19 @@ jini <- function(G,lambda) {
   list(beta=beta,se=se)
 } ## jini
 
+compress.iseq <- function(x) {
+## takes a set of non-negative integers and returns minimal code for generating it e.g.
+## c(3,2,6,7,1,9,8) is generated by "c(1:3,6:7,9:10)"
+  x1 <- sort(x)
+  br <- diff(x1)!=1 ## TRUE at sequence breaks
+  txt <- paste(x1[c(TRUE,br)],x1[c(br,TRUE)],sep=":") ## subsequences
+  txt1 <- paste(x1[c(TRUE,br)]) ## subseq starts
+  ii <- x1[c(TRUE,br)]==x1[c(br,TRUE)] ## index start and end equal
+  txt[ii] <- txt1[ii] ## replace length on sequences with integers
+  paste("c(",paste(txt,collapse=","),")",sep="")
+} ## compress.iseq
+
+
 jagam <- function(formula,family=gaussian,data=list(),file,weights=NULL,na.action,
 offset=NULL,knots=NULL,sp=NULL,drop.unused.levels=TRUE,control=gam.control(),centred=TRUE,
 sp.prior = "gamma",diagonalize=FALSE) {
@@ -165,7 +178,7 @@ sp.prior = "gamma",diagonalize=FALSE) {
   ## get initial values, for use by JAGS, and to guess suitable values for
   ## uninformative priors...
 
-  lambda <- initial.spg(G$X,G$y,G$w,family,G$S,G$off,offset=G$offset,L=G$L) ## initial sp values
+  lambda <- initial.spg(G$X,G$y,G$w,family,G$S,G$rank,G$off,offset=G$offset,L=G$L) ## initial sp values
   jags.ini <- list()
   lam <- if (is.null(G$L)) lambda else G$L%*%lambda
   jin <- jini(G,lam)
@@ -207,18 +220,19 @@ sp.prior = "gamma",diagonalize=FALSE) {
     }
     cat("  ## prior for ",G$smooth[[i]]$label,"... \n",file=file,append=TRUE,sep="")
     if (seperable) {
-      b0 <- G$smooth[[i]]$first.para
+      b0 <- G$smooth[[i]]$first.para;b1 <- G$smooth[[i]]$last.para
       if (M==0) {
         cat("  ## Note fixed vague prior, CHECK tau = 1/",signif(1/sqrt(ptau),2),"^2...\n",file=file,append=TRUE,sep="")
-        b1 <- G$smooth[[i]]$last.para
+        #b1 <- G$smooth[[i]]$last.para
         ptau <- min(prior.tau[b0:b1])
         cat("  for (i in ",b0,":",b1,") { b[i] ~ dnorm(0,",ptau,") }\n",file=file,append=TRUE,sep="")
       } else for (j in 1:M) {
         D <- diag(G$smooth[[i]]$S[[j]]) > 0
-        b1 <- sum(as.numeric(D)) + b0 - 1
+        #b1 <- sum(as.numeric(D)) + b0 - 1
         n.sp <- n.sp + 1
-        cat("  for (i in ",b0,":",b1,") { b[i] ~ dnorm(0, lambda[",n.sp,"]) }\n",file=file,append=TRUE,sep="")
-        b0 <- b1 + 1
+        #cat("  for (i in ",b0,":",b1,") { b[i] ~ dnorm(0, lambda[",n.sp,"]) }\n",file=file,append=TRUE,sep="")
+        #b0 <- b1 + 1
+	cat("  for (i in ",compress.iseq((b0:b1)[D]),") { b[i] ~ dnorm(0, lambda[",n.sp,"]) }\n",file=file,append=TRUE,sep="")
       }
     } else { ## inseperable - requires the penalty matrices to be supplied to JAGS... 
       b0 <- G$smooth[[i]]$first.para; b1 <- G$smooth[[i]]$last.para
